"use server";

import Docker from 'dockerode';

const docker = new Docker();

export default async function serverAction(sql: string, databaseType: string): Promise<string> {
  let dockerImage: string;
  let command: string;
  let env: string[] = [];

  switch (databaseType) {
    case 'sqlserver':
      dockerImage = 'mcr.microsoft.com/mssql/server:2019-latest';
      command = `sqlcmd -S localhost -U SA -P YourStrong!Passw0rd -Q "${sql}"`;
      env = ['ACCEPT_EULA=Y', 'SA_PASSWORD=YourStrong!Passw0rd'];
      break;
    case 'mysql':
      dockerImage = 'mysql:latest';
      command = `mysql -u root -pYourStrong!Passw0rd -e "${sql}"`;
      env = ['MYSQL_ROOT_PASSWORD=YourStrong!Passw0rd'];
      break;
    default:
      throw new Error('Unsupported database type');
  }

  try {
    // Start Docker container
    const container = await docker.run(dockerImage, [], process.stdout, {
      Tty: false,
      Env: env,
      HostConfig: {
        PortBindings: {
          '3306/tcp': [{ HostPort: '3306' }], // MySQL port binding
          '1433/tcp': [{ HostPort: '1433' }], // SQL Server port binding
        },
      },
    });

    // Wait for the database to start (this is a simple sleep, you might want a more robust solution)
    await new Promise(resolve => setTimeout(resolve, 10000));

    // Execute SQL command
    const exec = await container.exec({
      Cmd: ['/bin/sh', '-c', command],
      AttachStdout: true,
      AttachStderr: true,
    });

    const stream = await exec.start({});

    let output = '';
    stream.on('data', (chunk: { toString: () => string; }) => {
      output += chunk.toString();
    });

    await new Promise((resolve) => stream.on('end', resolve));

    // Stop and remove Docker container
    await container.stop();
    await container.remove();

    return output;
  } catch (error) {
    throw error;
  }
}